@@ -14,6 +14,10 @@
 #include "image.h"
 #include "stream.h"
 
+#ifdef Libisofs_with_checksumS
+#include "md5.h"
+#endif /*  Libisofs_with_checksumS */
+
 #include <stdlib.h>
 #include <string.h>
 #include <limits.h>
@@ -44,6 +48,11 @@ int iso_file_src_create(Ecma119Image *img, IsoFile *file, IsoFileSrc **src)
     dev_t dev_id;
     ino_t ino_id;
 
+#ifdef Libisofs_with_checksumS
+    int cret, no_md5= 0;
+    void *xipt = NULL;
+#endif
+
     if (img == NULL || file == NULL || src == NULL) {
         return ISO_NULL_POINTER;
     }
@@ -88,11 +97,54 @@ int iso_file_src_create(Ecma119Image *img, IsoFile *file, IsoFileSrc **src)
     /* insert the filesrc in the tree */
     ret = iso_rbtree_insert(img->files, fsrc, (void**)src);
     if (ret <= 0) {
+
+#ifdef Libisofs_with_checksumS
+
+        if (ret == 0 && (*src)->checksum_index > 0) {
+            /* Duplicate file source was mapped to previously registered source
+            */
+            cret = iso_file_set_isofscx(file, (*src)->checksum_index, 0);
+            if (cret < 0)
+                ret = cret;
+        }
+
+#endif /* Libisofs_with_checksumS */
+
         free(fsrc->sections);
         free(fsrc);
         return ret;
     }
     iso_stream_ref(fsrc->stream);
+
+#ifdef Libisofs_with_checksumS
+
+    if ((img->md5_file_checksums & 1) &&
+        file->from_old_session && img->appendable) {
+        ret = iso_node_get_xinfo((IsoNode *) file, checksum_md5_xinfo_func,
+                                  &xipt);
+        if (ret <= 0)
+            ret = iso_node_get_xinfo((IsoNode *) file, checksum_cx_xinfo_func,
+                                      &xipt);
+        if (ret <= 0)
+            /* Omit MD5 indexing with old image nodes which have no MD5 */
+            no_md5 = 1;
+    }
+
+    if ((img->md5_file_checksums & 1) && !no_md5) {
+        img->checksum_idx_counter++;
+        if (img->checksum_idx_counter < 0x80000000) {
+            fsrc->checksum_index = img->checksum_idx_counter;
+        } else {
+            fsrc->checksum_index= 0;
+            img->checksum_idx_counter= 0x80000000; /* keep from rolling over */
+        }
+        cret = iso_file_set_isofscx(file, (*src)->checksum_index, 0);
+        if (cret < 0)
+            return cret;
+    }
+
+#endif /* Libisofs_with_checksumS */
+
     return ISO_SUCCESS;
 }
 
@@ -241,41 +293,45 @@ int filesrc_close(IsoFileSrc *file)
 static
 int filesrc_read(IsoFileSrc *file, char *buf, size_t count)
 {
-    size_t bytes = 0;
+    size_t got;
 
-    /* loop to ensure the full buffer is filled */
-    do {
-        ssize_t result;
-        result = iso_stream_read(file->stream, buf + bytes, count - bytes);
-        if (result < 0) {
-            /* fill buffer with 0s and return */
-            memset(buf + bytes, 0, count - bytes);
-            return result;
-        }
-        if (result == 0)
-            break;
-        bytes += result;
-    } while (bytes < count);
-
-    if (bytes < count) {
-        /* eof */
-        memset(buf + bytes, 0, count - bytes);
-        return 0;
-    } else {
-        return 1;
-    }
+    return iso_stream_read_buffer(file->stream, buf, count, &got);
 }
 
+#ifdef Libisofs_with_checksumS
+
+/* @return 1=ok, md5 is valid,
+           0= not ok, go on,
+          <0 fatal error, abort 
+*/  
+
+static
+int filesrc_make_md5(Ecma119Image *t, IsoFileSrc *file, char md5[16], int flag)
+{
+    return iso_stream_make_md5(file->stream, md5, 0);
+}
+
+#endif /* Libisofs_with_checksumS */
+
 static
 int filesrc_writer_write_data(IsoImageWriter *writer)
 {
-    int res;
+    int res, ret, was_error;
     size_t i, b;
     Ecma119Image *t;
     IsoFileSrc *file;
     IsoFileSrc **filelist;
     char name[PATH_MAX];
     char buffer[BLOCK_SIZE];
+    off_t file_size;
+    uint32_t nblocks;
+
+#ifdef Libisofs_with_checksumS
+    void *ctx= NULL;
+    char md5[16], pre_md5[16];
+    int pre_md5_valid = 0;
+#endif
+
 
     if (writer == NULL) {
         return ISO_ASSERT_FAILURE;
@@ -288,8 +344,19 @@ int filesrc_writer_write_data(IsoImageWriter *writer)
 
     i = 0;
     while ((file = filelist[i++]) != NULL) {
+        was_error = 0;
+        file_size = iso_file_src_get_size(file);
+        nblocks = DIV_UP(file_size, BLOCK_SIZE);
+ 
+#ifdef Libisofs_with_checksumS
 
-        uint32_t nblocks = DIV_UP(iso_file_src_get_size(file), BLOCK_SIZE);
+        pre_md5_valid = 0; 
+        if (file->checksum_index > 0 && (t->md5_file_checksums & 2)) {
+            /* Obtain an MD5 of content by a first read pass */
+            pre_md5_valid = filesrc_make_md5(t, file, pre_md5, 0);
+        }
+
+#endif /* Libisofs_with_checksumS */
 
         res = filesrc_open(file);
         iso_stream_get_file_name(file->stream, name);
@@ -299,28 +366,33 @@ int filesrc_writer_write_data(IsoImageWriter *writer)
              * 0's to image
              */
             iso_report_errfile(name, ISO_FILE_CANT_WRITE, 0, 0);
+            was_error = 1;
             res = iso_msg_submit(t->image->id, ISO_FILE_CANT_WRITE, res,
                       "File \"%s\" can't be opened. Filling with 0s.", name);
             if (res < 0) {
-                return res; /* aborted due to error severity */
+                ret = res; /* aborted due to error severity */
+                goto ex;
             }
             memset(buffer, 0, BLOCK_SIZE);
             for (b = 0; b < nblocks; ++b) {
                 res = iso_write(t, buffer, BLOCK_SIZE);
                 if (res < 0) {
                     /* ko, writer error, we need to go out! */
-                    return res;
+                    ret = res;
+                    goto ex;
                 }
             }
             continue;
         } else if (res > 1) {
             iso_report_errfile(name, ISO_FILE_CANT_WRITE, 0, 0);
+            was_error = 1;
             res = iso_msg_submit(t->image->id, ISO_FILE_CANT_WRITE, 0,
                       "Size of file \"%s\" has changed. It will be %s", name,
                       (res == 2 ? "truncated" : "padded with 0's"));
             if (res < 0) {
                 filesrc_close(file);
-                return res; /* aborted due to error severity */
+                ret = res; /* aborted due to error severity */
+                goto ex;
             }
         }
 #ifdef LIBISOFS_VERBOSE_DEBUG
@@ -329,6 +401,18 @@ int filesrc_writer_write_data(IsoImageWriter *writer)
         }
 #endif
 
+ 
+#ifdef Libisofs_with_checksumS
+ 
+        if (file->checksum_index > 0) {
+            /* initialize file checksum */
+            res = iso_md5_start(&ctx);
+            if (res <= 0)
+                file->checksum_index = 0;
+        }
+
+#endif /* Libisofs_with_checksumS */
+ 
         /* write file contents to image */
         for (b = 0; b < nblocks; ++b) {
             int wres;
@@ -341,8 +425,25 @@ int filesrc_writer_write_data(IsoImageWriter *writer)
             if (wres < 0) {
                 /* ko, writer error, we need to go out! */
                 filesrc_close(file);
-                return wres;
+                ret = wres;
+                goto ex;
             }
+ 
+#ifdef Libisofs_with_checksumS
+
+            if (file->checksum_index > 0) {
+                /* Add to file checksum */
+                if (file_size - b * BLOCK_SIZE > BLOCK_SIZE)
+                    res = BLOCK_SIZE;
+                else
+                    res = file_size - b * BLOCK_SIZE;
+                res = iso_md5_compute(ctx, buffer, res);
+                if (res <= 0)
+                    file->checksum_index = 0;
+            }
+
+#endif /* Libisofs_with_checksumS */
+    
         }
 
         filesrc_close(file);
@@ -350,6 +451,7 @@ int filesrc_writer_write_data(IsoImageWriter *writer)
         if (b < nblocks) {
             /* premature end of file, due to error or eof */
             iso_report_errfile(name, ISO_FILE_CANT_WRITE, 0, 0);
+            was_error = 1;
             if (res < 0) {
                 /* error */
                 res = iso_msg_submit(t->image->id, ISO_FILE_CANT_WRITE, res,
@@ -361,7 +463,8 @@ int filesrc_writer_write_data(IsoImageWriter *writer)
             }
 
             if (res < 0) {
-                return res; /* aborted due error severity */
+                ret = res; /* aborted due error severity */
+                goto ex;
             }
 
             /* fill with 0s */
@@ -372,13 +475,67 @@ int filesrc_writer_write_data(IsoImageWriter *writer)
                 res = iso_write(t, buffer, BLOCK_SIZE);
                 if (res < 0) {
                     /* ko, writer error, we need to go out! */
-                    return res;
+                    ret = res;
+                    goto ex;
                 }
+ 
+#ifdef Libisofs_with_checksumS
+
+                if (file->checksum_index > 0) {
+                    /* Add to file checksum */
+                    if (file_size - b * BLOCK_SIZE > BLOCK_SIZE)
+                        res = BLOCK_SIZE;
+                    else
+                        res = file_size - b * BLOCK_SIZE;
+                    res = iso_md5_compute(ctx, buffer, res);
+                    if (res <= 0)
+                        file->checksum_index = 0;
+                }
+
+#endif /* Libisofs_with_checksumS */
+    
             }
         }
+
+#ifdef Libisofs_with_checksumS
+
+        if (file->checksum_index > 0) {
+            /* Obtain checksum and dispose checksum context */
+            res = iso_md5_end(&ctx, md5);
+            if (res <= 0)
+                file->checksum_index = 0;
+            if ((t->md5_file_checksums & 2) && pre_md5_valid > 0 &&
+                !was_error) {
+                if (! iso_md5_match(md5, pre_md5)) {
+                    /* Issue MISHAP event */
+                    iso_report_errfile(name, ISO_MD5_STREAM_CHANGE, 0, 0);
+                    was_error = 1;
+                    res = iso_msg_submit(t->image->id, ISO_MD5_STREAM_CHANGE,0,
+           "Content of file '%s' changed while it was written into the image.",
+                                         name);
+                    if (res < 0) {
+                        ret = res; /* aborted due to error severity */
+                        goto ex;
+                    }
+                }
+            }
+            /* Write md5 into checksum buffer at file->checksum_index */
+            memcpy(t->checksum_buffer + 16 * file->checksum_index, md5, 16);
+        }
+
+#endif /* Libisofs_with_checksumS */
+
     }
 
-    return ISO_SUCCESS;
+    ret = ISO_SUCCESS;
+ex:;
+
+#ifdef Libisofs_with_checksumS
+    if (ctx != NULL) /* avoid any memory leak */
+        iso_md5_end(&ctx, md5);
+#endif
+
+    return ret;
 }
 
 static
@@ -395,7 +552,7 @@ int iso_file_src_writer_create(Ecma119Image *target)
 
     writer = malloc(sizeof(IsoImageWriter));
     if (writer == NULL) {
-        return ISO_ASSERT_FAILURE;
+        return ISO_OUT_OF_MEM;
     }
 
     writer->compute_data_blocks = filesrc_writer_compute_data_blocks;
