@@ -246,11 +246,32 @@ time_t iso_datetime_read_17(const uint8_t *buf);
  */
 int iso_eaccess(const char *path);
 
+#ifdef NIX
+/* <<< Buggy and not used any more */
+
+/**
+ * Copy up to \p len chars from \p buf and return this newly allocated
+ * string. The new string is null-terminated.
+ * Note:
+ * Trailing blanks will be removed. But the first one of an all blank string
+ * persists. It is unclear whether this is a bug or a feature.
+ */
+char *strcopy(const char *buf, size_t len);
+
+#endif /* NIX */
+
 /**
  * Copy up to \p len chars from \p buf and return this newly allocated
  * string. The new string is null-terminated.
  */
-char *strcopy(const char *buf, size_t len);
+char *iso_util_strcopy(const char *buf, size_t len);
+
+/**
+ * Copy up to \p len chars from \p buf and return this newly allocated
+ * string. The new string is null-terminated.
+ * Any trailing blanks will be removed.
+ */
+char *iso_util_strcopy_untail(const char *buf, size_t len);
 
 /**
  * Copy up to \p max characters from \p src to \p dest. If \p src has less than
